@@ -135,6 +135,28 @@ ParseResult ParseRpcMessage(butil::IOBuf* source, Socket* socket,
     return MakeMessage(msg);
 }
 
+static bool SerializeResponse(const google::protobuf::Message& res,
+                              Controller& cntl, CompressType compress_type,
+                              butil::IOBuf& buf) {
+    if (res.GetDescriptor() == SerializedResponse::descriptor()) {
+        buf.swap(((SerializedResponse&)res).serialized_data());
+        return true;
+    }
+
+    if (!res.IsInitialized()) {
+        cntl.SetFailed(ERESPONSE,
+                       "Missing required fields in response: %s",
+                       res.InitializationErrorString().c_str());
+        return false;
+    } else if (!SerializeAsCompressedData(res, &buf, compress_type)) {
+        cntl.SetFailed(ERESPONSE,
+                       "Fail to serialize response, CompressType=%s",
+                       CompressTypeToCStr(compress_type));
+        return false;
+    }
+    return true;
+}
+
 // Used by UT, can't be static.
 void SendRpcResponse(int64_t correlation_id,
                      Controller* cntl, 
@@ -170,18 +192,10 @@ void SendRpcResponse(int64_t correlation_id,
     // `res' can be NULL here, in which case we don't serialize it
     // If user calls `SetFailed' on Controller, we don't serialize
     // response either
-    CompressType type = cntl->response_compress_type();
+    CompressType compress_type = cntl->response_compress_type();
     if (res != NULL && !cntl->Failed()) {
-        if (!res->IsInitialized()) {
-            cntl->SetFailed(
-                ERESPONSE, "Missing required fields in response: %s", 
-                res->InitializationErrorString().c_str());
-        } else if (!SerializeAsCompressedData(*res, &res_body, type)) {
-            cntl->SetFailed(ERESPONSE, "Fail to serialize response, "
-                            "CompressType=%s", CompressTypeToCStr(type));
-        } else {
-            append_body = true;
-        }
+        append_body = SerializeResponse(
+            *res, *cntl, compress_type, res_body);
     }
 
     // Don't use res->ByteSize() since it may be compressed
@@ -207,7 +221,7 @@ void SendRpcResponse(int64_t correlation_id,
         response_meta->set_error_text(cntl->ErrorText());
     }
     meta.set_correlation_id(correlation_id);
-    meta.set_compress_type(cntl->response_compress_type());
+    meta.set_compress_type(compress_type);
     if (attached_size > 0) {
         meta.set_attachment_size(attached_size);
     }
@@ -236,7 +250,7 @@ void SendRpcResponse(int64_t correlation_id,
     SerializeRpcHeaderAndMeta(&res_buf, meta, res_size + attached_size);
     if (append_body) {
         res_buf.append(res_body.movable());
-        if (attached_size) {
+        if (attached_size > 0) {
             res_buf.append(cntl->response_attachment().movable());
         }
     }
@@ -360,6 +374,7 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
         LOG(WARNING) << "Fail to new Controller";
         return;
     }
+
     std::unique_ptr<google::protobuf::Message> req;
     std::unique_ptr<google::protobuf::Message> res;
 
@@ -442,84 +457,129 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
             break;
         }
 
-        // NOTE(gejun): jprotobuf sends service names without packages. So the
-        // name should be changed to full when it's not.
-        butil::StringPiece svc_name(request_meta.service_name());
-        if (svc_name.find('.') == butil::StringPiece::npos) {
-            const Server::ServiceProperty* sp =
-                server_accessor.FindServicePropertyByName(svc_name);
-            if (NULL == sp) {
-                cntl->SetFailed(ENOSERVICE, "Fail to find service=%s",
-                                request_meta.service_name().c_str());
-                break;
-            }
-            svc_name = sp->service->GetDescriptor()->full_name();
-        }
-        const Server::MethodProperty* mp =
-            server_accessor.FindMethodPropertyByFullName(
-                svc_name, request_meta.method_name());
-        if (NULL == mp) {
-            cntl->SetFailed(ENOMETHOD, "Fail to find method=%s/%s",
-                            request_meta.service_name().c_str(),
-                            request_meta.method_name().c_str());
-            break;
-        } else if (mp->service->GetDescriptor()
-                   == BadMethodService::descriptor()) {
-            BadMethodRequest breq;
-            BadMethodResponse bres;
-            breq.set_service_name(request_meta.service_name());
-            mp->service->CallMethod(mp->method, cntl.get(), &breq, &bres, NULL);
-            break;
-        }
-        // Switch to service-specific error.
-        non_service_error.release();
-        method_status = mp->status;
-        if (method_status) {
-            int rejected_cc = 0;
-            if (!method_status->OnRequested(&rejected_cc, cntl.get())) {
-                cntl->SetFailed(ELIMIT, "Rejected by %s's ConcurrencyLimiter, concurrency=%d",
-                                mp->method->full_name().c_str(), rejected_cc);
-                break;
-            }
-        }
-        google::protobuf::Service* svc = mp->service;
-        const google::protobuf::MethodDescriptor* method = mp->method;
-        accessor.set_method(method);
-
-
-        if (!server->AcceptRequest(cntl.get())) {
-            break;
-        }
-
-        if (span) {
-            span->ResetServerSpanName(method->full_name());
-        }
         const int req_size = static_cast<int>(msg->payload.size());
-        butil::IOBuf req_buf;
-        butil::IOBuf* req_buf_ptr = &msg->payload;
         if (meta.has_attachment_size()) {
             if (req_size < meta.attachment_size()) {
                 cntl->SetFailed(EREQUEST,
                     "attachment_size=%d is larger than request_size=%d",
-                     meta.attachment_size(), req_size);
+                    meta.attachment_size(), req_size);
                 break;
             }
-            int body_without_attachment_size = req_size - meta.attachment_size();
-            msg->payload.cutn(&req_buf, body_without_attachment_size);
-            req_buf_ptr = &req_buf;
-            cntl->request_attachment().swap(msg->payload);
         }
 
-        CompressType req_cmp_type = (CompressType)meta.compress_type();
-        req.reset(svc->GetRequestPrototype(method).New());
-        if (!ParseFromCompressedData(*req_buf_ptr, req.get(), req_cmp_type)) {
-            cntl->SetFailed(EREQUEST, "Fail to parse request message, "
-                            "CompressType=%s, request_size=%d", 
-                            CompressTypeToCStr(req_cmp_type), req_size);
-            break;
+        google::protobuf::Service* svc = NULL;
+        google::protobuf::MethodDescriptor* method = NULL;
+        if (NULL != server->options().baidu_master_service) {
+            svc = server->options().baidu_master_service;
+            auto sampled_request = new (std::nothrow) SampledRequest;
+            if (NULL == sampled_request) {
+                cntl->SetFailed(ENOMEM, "Fail to get sampled_request");
+                break;
+            }
+            sampled_request->meta.set_service_name(request_meta.service_name());
+            sampled_request->meta.set_method_name(request_meta.method_name());
+            cntl->reset_sampled_request(sampled_request);
+            // Switch to service-specific error.
+            non_service_error.release();
+            method_status = server->options().baidu_master_service->_status;
+            if (method_status) {
+                int rejected_cc = 0;
+                if (!method_status->OnRequested(&rejected_cc, cntl.get())) {
+                    cntl->SetFailed(
+                        ELIMIT,
+                        "Rejected by %s's ConcurrencyLimiter, concurrency=%d",
+                        butil::class_name<BaiduMasterService>(), rejected_cc);
+                    break;
+                }
+            }
+            if (span) {
+                span->ResetServerSpanName(sampled_request->meta.method_name());
+            }
+
+            auto serialized_request = (SerializedRequest*)
+                svc->GetRequestPrototype(NULL).New();
+            req.reset(serialized_request);
+            res.reset(svc->GetResponsePrototype(NULL).New());
+
+            msg->payload.cutn(&serialized_request->serialized_data(),
+                              req_size - meta.attachment_size());
+            if (!msg->payload.empty()) {
+                cntl->request_attachment().swap(msg->payload);
+            }
+        } else {
+            // NOTE(gejun): jprotobuf sends service names without packages. So the
+            // name should be changed to full when it's not.
+            butil::StringPiece svc_name(request_meta.service_name());
+            if (svc_name.find('.') == butil::StringPiece::npos) {
+                const Server::ServiceProperty* sp =
+                    server_accessor.FindServicePropertyByName(svc_name);
+                if (NULL == sp) {
+                    cntl->SetFailed(ENOSERVICE, "Fail to find service=%s",
+                        request_meta.service_name().c_str());
+                    break;
+                }
+                svc_name = sp->service->GetDescriptor()->full_name();
+            }
+            const Server::MethodProperty* mp =
+                server_accessor.FindMethodPropertyByFullName(
+                    svc_name, request_meta.method_name());
+            if (NULL == mp) {
+                cntl->SetFailed(ENOMETHOD, "Fail to find method=%s/%s",
+                                request_meta.service_name().c_str(),
+                                request_meta.method_name().c_str());
+                break;
+            } else if (mp->service->GetDescriptor() == BadMethodService::descriptor()) {
+                BadMethodRequest breq;
+                BadMethodResponse bres;
+                breq.set_service_name(request_meta.service_name());
+                mp->service->CallMethod(mp->method, cntl.get(), &breq, &bres, NULL);
+                break;
+            }
+            // Switch to service-specific error.
+            non_service_error.release();
+            method_status = mp->status;
+            if (method_status) {
+                int rejected_cc = 0;
+                if (!method_status->OnRequested(&rejected_cc, cntl.get())) {
+                    cntl->SetFailed(
+                        ELIMIT,
+                        "Rejected by %s's ConcurrencyLimiter, concurrency=%d",
+                        mp->method->full_name().c_str(), rejected_cc);
+                    break;
+                }
+            }
+            svc = mp->service;
+            method = const_cast<google::protobuf::MethodDescriptor*>(mp->method);
+            accessor.set_method(method);
+
+            if (span) {
+                span->ResetServerSpanName(method->full_name());
+            }
+
+            if (!server->AcceptRequest(cntl.get())) {
+                break;
+            }
+
+            butil::IOBuf req_buf;
+            int body_without_attachment_size = req_size - meta.attachment_size();
+            msg->payload.cutn(&req_buf, body_without_attachment_size);
+            if (meta.attachment_size() > 0) {
+                cntl->request_attachment().swap(msg->payload);
+            }
+
+            auto req_cmp_type = static_cast<CompressType>(meta.compress_type());
+            req.reset(svc->GetRequestPrototype(method).New());
+            if (!ParseFromCompressedData(req_buf, req.get(), req_cmp_type)) {
+                cntl->SetFailed(EREQUEST, "Fail to parse request message, "
+                                          "CompressType=%s, request_size=%d",
+                                CompressTypeToCStr(req_cmp_type), req_size);
+                break;
+            }
+
+            res.reset(svc->GetResponsePrototype(method).New());
+            req_buf.clear();
         }
-        
-        res.reset(svc->GetResponsePrototype(method).New());
+
         // `socket' will be held until response has been sent
         google::protobuf::Closure* done = ::brpc::NewCallback<
             int64_t, Controller*, const google::protobuf::Message*,
@@ -531,7 +591,6 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
 
         // optional, just release resource ASAP
         msg.reset();
-        req_buf.clear();
 
         if (span) {
             span->set_start_callback_us(butil::cpuwide_time_us());
@@ -653,10 +712,13 @@ void ProcessRpcResponse(InputMessageBase* msg_base) {
             cntl->response_attachment().swap(msg->payload);
         }
 
-        const CompressType res_cmp_type = (CompressType)meta.compress_type();
+        auto res_cmp_type = (CompressType)meta.compress_type();
         cntl->set_response_compress_type(res_cmp_type);
         if (cntl->response()) {
-            if (!ParseFromCompressedData(
+            if (cntl->response()->GetDescriptor() == SerializedResponse::descriptor()) {
+                ((SerializedResponse*)cntl->response())->
+                    serialized_data().append(*res_buf_ptr);
+            } else if (!ParseFromCompressedData(
                     *res_buf_ptr, cntl->response(), res_cmp_type)) {
                 cntl->SetFailed(
                     ERESPONSE, "Fail to parse response message, "
@@ -692,13 +754,15 @@ void PackRpcRequest(butil::IOBuf* req_buf,
                                        method->service()->name());
         request_meta->set_method_name(method->name());
         meta.set_compress_type(cntl->request_compress_type());
-    } else if (cntl->sampled_request()) {
+    } else if (NULL != cntl->sampled_request()) {
         // Replaying. Keep service-name as the one seen by server.
         request_meta->set_service_name(cntl->sampled_request()->meta.service_name());
         request_meta->set_method_name(cntl->sampled_request()->meta.method_name());
-        meta.set_compress_type(cntl->sampled_request()->meta.compress_type());
+        meta.set_compress_type(cntl->sampled_request()->meta.has_compress_type() ?
+                               cntl->sampled_request()->meta.compress_type() :
+                               cntl->request_compress_type());
     } else {
-        return cntl->SetFailed(ENOMETHOD, "%s.method is NULL", __FUNCTION__);
+        return cntl->SetFailed(ENOMETHOD, "%s.method is NULL", __func__ );
     }
     if (cntl->has_log_id()) {
         request_meta->set_log_id(cntl->log_id());
