@@ -42,6 +42,8 @@
 #include "brpc/http2.h"
 #include "brpc/redis.h"
 #include "brpc/interceptor.h"
+#include "brpc/concurrency_limiter.h"
+#include "brpc/baidu_master_service.h"
 
 namespace brpc {
 
@@ -224,6 +226,8 @@ struct ServerOptions {
     // Default: false
     bool use_rdma;
 
+    BaiduMasterService* baidu_master_service;
+
     // [CAUTION] This option is for implementing specialized http proxies,
     // most users don't need it. Don't change this option unless you fully
     // understand the description below.
@@ -234,7 +238,7 @@ struct ServerOptions {
     // and response must have no fields.
     //
     // Owned by Server and deleted in server's destructor
-    google::protobuf::Service* http_master_service;
+    ::google::protobuf::Service* http_master_service;
 
     // If this field is on, contents on /health page is generated by calling
     // health_reporter->GenerateReport(). This object is NOT owned by server
@@ -674,6 +678,26 @@ friend class Controller;
     AdaptiveMaxConcurrency& MaxConcurrencyOf(MethodProperty*);
     int MaxConcurrencyOf(const MethodProperty*) const;
 
+    static bool CreateConcurrencyLimiter(const AdaptiveMaxConcurrency& amc,
+                                         ConcurrencyLimiter** out);
+
+    template <typename T>
+    int SetServiceMaxConcurrency(T* service) {
+        if (NULL != service) {
+            const AdaptiveMaxConcurrency* amc = &service->_max_concurrency;
+            if (amc->type() == AdaptiveMaxConcurrency::UNLIMITED()) {
+                amc = &_options.method_max_concurrency;
+            }
+            ConcurrencyLimiter* cl = NULL;
+            if (!CreateConcurrencyLimiter(*amc, &cl)) {
+                LOG(ERROR) << "Fail to create ConcurrencyLimiter for method";
+                return -1;
+            }
+            service->_status->SetConcurrencyLimiter(cl);
+        }
+        return 0;
+    }
+
     DISALLOW_COPY_AND_ASSIGN(Server);
 
     // Put frequently-accessed data pool at first.
