@@ -26,6 +26,9 @@
 #include <butil/macros.h>
 #include <butil/fd_guard.h>
 #include <butil/files/scoped_file.h>
+#include <brpc/policy/baidu_rpc_meta.pb.h>
+#include <brpc/policy/baidu_rpc_protocol.h>
+#include <brpc/policy/most_common_message.h>
 #include "brpc/global.h"
 #include "brpc/socket.h"
 #include "brpc/server.h"
@@ -54,11 +57,11 @@ const std::string EXP_RESPONSE = "world";
 class EchoServiceImpl : public test::EchoService {
 public:
     EchoServiceImpl() : count(0) {}
-    virtual ~EchoServiceImpl() { g_delete = true; }
-    virtual void Echo(google::protobuf::RpcController* cntl_base,
-                      const test::EchoRequest* request,
-                      test::EchoResponse* response,
-                      google::protobuf::Closure* done) {
+    ~EchoServiceImpl() override { g_delete = true; }
+    void Echo(google::protobuf::RpcController* cntl_base,
+              const test::EchoRequest* request,
+              test::EchoResponse* response,
+              google::protobuf::Closure* done) override {
         brpc::ClosureGuard done_guard(done);
         brpc::Controller* cntl = (brpc::Controller*)cntl_base;
         count.fetch_add(1, butil::memory_order_relaxed);
@@ -207,7 +210,7 @@ TEST_F(SSLTest, force_ssl) {
         test::EchoService_Stub stub(&channel);
         test::EchoResponse res;
         stub.Echo(&cntl, &req, &res, NULL);
-        EXPECT_EQ(EXP_RESPONSE, res.message()) << cntl.ErrorText();
+        ASSERT_EQ(EXP_RESPONSE, res.message()) << cntl.ErrorText();
     }
 
     {
@@ -218,13 +221,98 @@ TEST_F(SSLTest, force_ssl) {
         test::EchoService_Stub stub(&channel);
         test::EchoResponse res;
         stub.Echo(&cntl, &req, &res, NULL);
-        EXPECT_TRUE(cntl.Failed());
+        ASSERT_TRUE(cntl.Failed());
     }
 
     ASSERT_EQ(0, server.Stop(0));
     ASSERT_EQ(0, server.Join());
 }
 
+void ProcessResponse(brpc::InputMessageBase* msg_base) {
+    brpc::DestroyingPtr<brpc::policy::MostCommonMessage> msg(
+        static_cast<brpc::policy::MostCommonMessage*>(msg_base));
+    brpc::policy::RpcMeta meta;
+    ASSERT_TRUE(brpc::ParsePbFromIOBuf(&meta, msg->meta));
+    const brpc::policy::RpcResponseMeta &response_meta = meta.response();
+    ASSERT_EQ(0, response_meta.error_code()) << response_meta.error_text();
+
+    const brpc::CallId cid = { static_cast<uint64_t>(meta.correlation_id()) };
+    brpc::Controller* cntl = NULL;
+    ASSERT_EQ(0, bthread_id_lock(cid, (void**)&cntl));
+    ASSERT_NE(nullptr, cntl);
+    ASSERT_TRUE(brpc::ParsePbFromIOBuf(cntl->response(), msg->payload));
+    ASSERT_EQ(0, bthread_id_unlock_and_destroy(cid));
+}
+
+TEST_F(SSLTest, connect_on_create) {
+    brpc::Protocol dummy_protocol = {
+        brpc::policy::ParseRpcMessage, brpc::SerializeRequestDefault,
+        brpc::policy::PackRpcRequest,NULL, ProcessResponse,
+        NULL, NULL, NULL, brpc::CONNECTION_TYPE_ALL, "ssl_ut_baidu"
+    };
+    ASSERT_EQ(0, RegisterProtocol((brpc::ProtocolType)30, dummy_protocol));
+
+    brpc::InputMessageHandler dummy_handler ={
+        dummy_protocol.parse, dummy_protocol.process_response,
+        NULL, NULL, dummy_protocol.name
+    };
+    brpc::InputMessenger messenger;
+    ASSERT_EQ(0, messenger.AddHandler(dummy_handler));
+
+    const int port = 8613;
+    brpc::Server server;
+    brpc::ServerOptions server_options;
+    server_options.force_ssl = true;
+
+    brpc::CertInfo cert;
+    cert.certificate = "cert1.crt";
+    cert.private_key = "cert1.key";
+    server_options.mutable_ssl_options()->default_cert = cert;
+
+    EchoServiceImpl echo_svc;
+    ASSERT_EQ(0, server.AddService(
+        &echo_svc, brpc::SERVER_DOESNT_OWN_SERVICE));
+    ASSERT_EQ(0, server.Start(port, &server_options));
+
+    // Create client socket.
+    brpc::SocketOptions socket_options;
+    butil::EndPoint ep(butil::IP_ANY, port);
+    socket_options.remote_side = ep;
+    socket_options.connect_on_create = true;
+    socket_options.on_edge_triggered_events = brpc::InputMessenger::OnNewMessages;
+    socket_options.user = &messenger;
+    brpc::ChannelSSLOptions ssl_options;
+    SSL_CTX* raw_ctx = brpc::CreateClientSSLContext(ssl_options);
+    ASSERT_NE(nullptr, raw_ctx);
+    std::shared_ptr<brpc::SocketSSLContext> ssl_ctx
+        = std::make_shared<brpc::SocketSSLContext>();
+    ssl_ctx->raw_ctx = raw_ctx;
+    socket_options.initial_ssl_ctx = ssl_ctx;
+
+    brpc::SocketId socket_id;
+    ASSERT_EQ(0, brpc::Socket::Create(socket_options, &socket_id));
+    brpc::SocketUniquePtr ptr;
+    ASSERT_EQ(0, brpc::Socket::Address(socket_id, &ptr));
+
+    test::EchoRequest req;
+    req.set_message(EXP_REQUEST);
+    for (int i = 0; i < 100; ++i) {
+        test::EchoResponse res;
+        butil::IOBuf request_buf;
+        butil::IOBuf request_body;
+        brpc::Controller cntl;
+        cntl._response = &res;
+        const brpc::CallId correlation_id = cntl.call_id();
+        brpc::SerializeRequestDefault(&request_body, &cntl, &req);
+        brpc::policy::PackRpcRequest(&request_buf, NULL, correlation_id.value,
+                                     test::EchoService_Stub::descriptor()->method(0),
+                                     &cntl, request_body, NULL);
+        ASSERT_EQ(0, ptr->Write(&request_buf));
+        brpc::Join(correlation_id);
+        ASSERT_EQ(EXP_RESPONSE, res.message());
+    }
+}
+
 void CheckCert(const char* cname, const char* cert) {
     const int port = 8613;
     brpc::Channel channel;
